<?php
/*
  Plugin Name: Payment gateway via Borgun SecurePay for WooCommerce
  Plugin URI: https://profiles.wordpress.org/tacticais/
  Description: Extends WooCommerce with a <a href="https://docs.borgun.is/hostedpayments/securepay/" target="_blank">Borgun SecurePay</a> gateway.
  Version: 1.3.19
  Author: Tactica
  Author URI: http://tactica.is
  Text Domain: borgun_woocommerce
  Domain Path: /languages
  Requires at least: 4.4
  Tested up to: 5.6
  WC tested up to: 4.7.1
  WC requires at least: 3.2.3
  License: GNU General Public License v3.0
  License URI: https://www.gnu.org/licenses/gpl-3.0.html
 */
define( 'BORGUN_DIR', plugin_dir_path( __FILE__ ) );
define( 'BORGUN_URL', plugin_dir_url( __FILE__ ) );
define( 'BORGUN_VERSION', '1.3.19' );
function borgun_wc_active() {
	if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
		return true;
	} else {
		return false;
	}
}

add_action( 'plugins_loaded', 'woocommerce_borgun_init', 0 );
add_action( 'plugins_loaded', 'woocommerce_borgun_textdomain' );
function woocommerce_borgun_init() {
	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}
	//Add the gateway to woocommerce
	add_filter( 'woocommerce_payment_gateways', 'add_borgun_gateway' );
	function add_borgun_gateway( $methods ) {
		$methods[] = 'WC_Gateway_Borgun';

		return $methods;
	}

	/**
	 * @property string testmode
	 * @property string merchantid
	 * @property string paymentgatewayid
	 * @property string secretkey
	 * @property string langpaymentpage
	 * @property string successurl
	 * @property string cancelurl
	 * @property string errorurl
	 * @property string notification_email
	 */
	class WC_Gateway_Borgun extends WC_Payment_Gateway {

		const BORGUN_ENDPOINT_SANDBOX = 'https://test.borgun.is/SecurePay/default.aspx';
		const BORGUN_ENDPOINT_LIVE = 'https://securepay.borgun.is/securepay/default.aspx';

		public function __construct() {
			$this->id                 = 'borgun';
			$this->icon               = BORGUN_URL . '/cards.png';
			$this->has_fields         = false;
			$this->method_title       =  __('Borgun', 'borgun_woocommerce');
			$this->method_description = __('Borgun Secure Payment Page enables merchants to sell products securely on the web with minimal integration effort', 'borgun_woocommerce');
			// Load the form fields
			$this->init_form_fields();
			$this->init_settings();
			// Get setting values
			$this->enabled            = $this->get_option( 'enabled' );
			$this->title              = $this->get_option( 'title' );
			$this->description        = $this->get_option( 'description' );
			$this->testmode           = $this->get_option( 'testmode' );
			$this->merchantid         = $this->get_option( 'merchantid' );
			$this->paymentgatewayid   = $this->get_option( 'paymentgatewayid' );
			$this->secretkey          = $this->get_option( 'secretkey' );
			$this->langpaymentpage    = $this->get_option( 'langpaymentpage' );
			$this->receipttext 		  = $this->get_option( 'receipttext', __('Thank you - your order is now pending payment. We are now redirecting you to Borgun to make payment.', 'borgun_woocommerce') );
			$this->redirecttext 	  = $this->get_option( 'redirecttext',  __('Thank you for your order. We are now redirecting you to Borgun to make payment.', 'borgun_woocommerce') );
			$this->successurl         = $this->get_option( 'successurl' );
			$this->cancelurl          = $this->get_option( 'cancelurl' );
			$this->errorurl           = $this->get_option( 'errorurl' );
			$this->notification_email = $this->get_option( 'notification_email' );
			// Filters
			add_filter( 'wcml_gateway_text_keys_to_translate', array( $this, 'borgun_text_keys_to_translate' ) );
			// Hooks
			add_action( 'woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_api_wc_gateway_borgun', array( $this, 'check_borgun_response' ) );
			add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'check_borgun_response' ) );
			add_action( 'before_woocommerce_pay', array( $this, 'checkout_payment_handler'), 9 );
			if ( ! $this->is_valid_for_use() ) {
				$this->enabled = false;
			}
		}

		public function admin_options() {
			?>
			<h3> <?php _e( 'Borgun', 'borgun_woocommerce' ); ?></h3>
			<p> <?php _e( 'Pay with your credit card via Borgun.', 'borgun_woocommerce' ); ?></p>
			<?php if ( $this->is_valid_for_use() ) : ?>
				<table class="form-table"><?php $this->generate_settings_html(); ?></table>
			<?php else : ?>
				<div class="inline error"><p><strong><?php _e( 'Gateway Disabled:', 'borgun_woocommerce' ); ?></strong><?php _e( 'Current Store currency is not valid for borgun gateway. Must be in ISK, USD, EUR, GBP, DKK, NOK, SEK, CHF, JPY, CAD, HUF', 'borgun_woocommerce' ); ?></p></div>
				<?php
			endif;
		}

		//Check if this gateway is enabled and available in the user's country
		function is_valid_for_use() {
			if ( ! in_array( get_woocommerce_currency(), array(
				'ISK',
				'USD',
				'EUR',
				'GBP',
				'DKK',
				'NOK',
				'SEK',
				'CHF',
				'JPY',
				'CAD',
				'HUF'
			) )
			) {
				return false;
			}

			return true;
		}

		//Initialize Gateway Settings Form Fields
		function init_form_fields() {
			$this->form_fields = array(
				'enabled'            => array(
					'title'       => __( 'Enable/Disable', 'borgun_woocommerce' ),
					'label'       => __( 'Enable Borgun', 'borgun_woocommerce' ),
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title'              => array(
					'title'       =>  __( 'Title', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' =>  __( 'This controls the title which the user sees during checkout.', 'borgun_woocommerce' ),
					'default'     =>  __( 'Borgun', 'borgun_woocommerce' )
				),
				'description'        => array(
					'title'       => __( 'Description', 'borgun_woocommerce' ),
					'type'        => 'textarea',
					'description' => __( 'This controls the description which the user sees during checkout.', 'borgun_woocommerce' ),
					'default'     => __( 'Pay with your credit card via Borgun.', 'borgun_woocommerce' )
				),
				'testmode'           => array(
					'title'       => __( 'Borgun Test Mode', 'borgun_woocommerce' ),
					'label'       => __( 'Enable Test Mode', 'borgun_woocommerce' ),
					'type'        => 'checkbox',
					'description' => __( 'Place the payment gateway in development mode.', 'borgun_woocommerce' ),
					'default'     => 'no'
				),
				'paymentgatewayid'   => array(
					'title'       => __( 'Payment Gateway ID', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' => __( 'This is the Payment Gateway ID supplied by Borgun.', 'borgun_woocommerce' ),
					'default'     => '16'
				),
				'merchantid'         => array(
					'title'       => __( 'Merchant ID', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' =>  __( 'This is the ID supplied by Borgun.', 'borgun_woocommerce' ),
					'default'     => '9275444'
				),
				'secretkey'          => array(
					'title'       => __( 'Secret Key', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' => __( 'This is the Secret Key supplied by Borgun.', 'borgun_woocommerce' ),
					'default'     => '99887766'
				),
				'notification_email' => array(
					'title'       => __( 'Notification Email', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' => __( 'This is the email Borgun will send payment receipts to.', 'borgun_woocommerce' ),
					'default'     => get_option( 'admin_email' )
				),
				'langpaymentpage'    => array(
					'title'       => __( 'Language of Payment Page', 'borgun_woocommerce' ),
					'type'        => 'select',
					'description' => __('Select which language to show on Payment Page.', 'borgun_woocommerce' ),
					'default'     => 'en',
					'options'     => array(
						'is' => __('Icelandic', 'borgun_woocommerce' ),
						'en' => __('English', 'borgun_woocommerce' ),
						'de' => __('German', 'borgun_woocommerce' ),
						'fr' => __('French', 'borgun_woocommerce' ),
						'it' => __('Italian', 'borgun_woocommerce' ),
						'pt' => __('Portugese', 'borgun_woocommerce' ),
						'ru' => __('Russian', 'borgun_woocommerce' ),
						'es' => __('Spanish', 'borgun_woocommerce' ),
						'se' => __('Swedish', 'borgun_woocommerce' ),
						'hu' => __('Hungarian', 'borgun_woocommerce' ),
						'si' => __('Slovene', 'borgun_woocommerce' )
					)
				),
				'receipttext'     	=> array(
					'title'       =>  __('Receipt text', 'borgun_woocommerce'),
					'type'        => 'textarea',
					'description' =>  __('Buyer will see this text after woocommerce order create.', 'borgun_woocommerce'),
					'default'     =>  __('Thank you - your order is now pending payment. We are now redirecting you to Borgun to make payment.', 'borgun_woocommerce'),
				),
				'redirecttext'     	=> array(
					'title'       =>  __('Redirect text', 'borgun_woocommerce'),
					'type'        => 'textarea',
					'description' => __('Buyer will see this text before redirecting to Borgun', 'borgun_woocommerce'),
					'default'     => __('Thank you for your order. We are now redirecting you to Borgun to make payment.', 'borgun_woocommerce'),
				),
				'successurl'         => array(
					'title'       =>  __('Success Page URL', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' =>  __('Buyer will be sent to this page after a successful payment.', 'borgun_woocommerce' ),
					'default'     => ''
				),
				'cancelurl'          => array(
					'title'       =>  __('Cancel Page URL', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' =>  __('Buyer will be sent to this page if he pushes the cancel button instead of finalizing the payment.', 'borgun_woocommerce' ),
					'default'     => ''
				),
				'errorurl'           => array(
					'title'       =>  __('Error Page URL', 'borgun_woocommerce' ),
					'type'        => 'text',
					'description' =>  __('Buyer will be sent to this page if an unexpected error occurs.', 'borgun_woocommerce' ),
					'default'     => ''
				),
			);
		}

		/**
		 * @param WC_Order $order
		 *
		 * @return false|string
		 */
		function check_hash( $order ) {
			$ipnUrl           = WC()->api_request_url( 'WC_Gateway_Borgun' );
			$hash             = array();
			$hash[]           = $this->merchantid;
			$hash[]           = esc_url_raw( $this->get_return_url( $order ) );
			$hash[]           = $ipnUrl;
			$hash[]           = 'WC-' . $order->get_id();
			$hash[]           = number_format( $order->get_total(), wc_get_price_decimals(), '.', '' );
			$hash[]           = $order->get_currency();
			$message          = implode( '|', $hash );
			$CheckHashMessage = utf8_encode( trim( $message ) );
			$Checkhash        = hash_hmac( 'sha256', $CheckHashMessage, $this->secretkey );

			return $Checkhash;
		}

		/**
		 * @param WC_Order $order
		 *
		 * @return false|string
		 */
		function check_order_hash( $order ) {
			$hash             = array();
			$hash[]           = 'WC-' . $order->get_id();
			$hash[]           = number_format( $order->get_total(), wc_get_price_decimals(), '.', '' );
			$hash[]           = $order->get_currency();
			$message          = implode( '|', $hash );
			$CheckHashMessage = utf8_encode( trim( $message ) );
			$Checkhash        = hash_hmac( 'sha256', $CheckHashMessage, $this->secretkey );

			return $Checkhash;
		}

		/**
		 * @param WC_Order $order
		 *
		 * @return array
		 */
		function get_borgun_args( $order ) {
			//Borgun Args
			global $wp_version;
			$ipnUrl = WC()->api_request_url( 'WC_Gateway_Borgun' );
			$borgun_args = array(
				'merchantid'             => $this->merchantid,
				'paymentgatewayid'       => $this->paymentgatewayid,
				'checkhash'              => $this->check_hash( $order ),
				'orderid'                => 'WC-' . $order->get_id(),
				'reference'              => $order->get_order_number(),
				'currency'               => $order->get_currency(),
				'language'               => $this->langpaymentpage,
				'SourceSystem'           => 'WP' . $wp_version . ' - WC' . WC()->version . ' - BRG' . BORGUN_VERSION,
				'buyeremail'             => $order->get_billing_email(),
				'returnurlsuccess'       => esc_url_raw($this->get_return_url( $order )),
				'returnurlsuccessserver' => $ipnUrl,
				'returnurlcancel'        =>$order->get_checkout_payment_url( true ),
				'returnurlerror'         => $order->get_checkout_payment_url( true ),
				'amount'                 => number_format( $order->get_total(), wc_get_price_decimals(), '.', '' ),
				'pagetype'               => '0',
				//If set as 1 then cardholder is required to insert email,mobile number,address.
				'skipreceiptpage'        => '1',
				'merchantemail'          => $this->notification_email,
			);
			// Cart Contents

			$include_tax = $this->tax_display();

			$item_loop = 0;
			if ( sizeof( $order->get_items( array( 'line_item', 'fee' ) ) ) > 0 ) {
				foreach ( $order->get_items( array( 'line_item', 'fee' ) ) as $item ) {
					if ( 'fee' === $item['type'] ) {
						$fee = $item->get_total();
						if ( $include_tax && $this->fee_tax_display($item) ){
							$fee += $item->get_total_tax();
						}
						$fee_total = $this->round( $fee, $order );
						$item_name = strip_tags( $item->get_name() );
						$borgun_args[ 'itemdescription_' . $item_loop ] = html_entity_decode( $item_name, ENT_NOQUOTES, 'UTF-8' );
						$borgun_args[ 'itemcount_' . $item_loop ]       = 1;
						$borgun_args[ 'itemunitamount_' . $item_loop ]  = $fee_total;
						$borgun_args[ 'itemamount_' . $item_loop ]      = $fee_total;

						$item_loop ++;
					}
					if ( $item['qty'] ) {
						$item_name = $item['name'];
						if ( $meta = wc_display_item_meta( $item ) ) {
							$item_name .= ' ( ' . $meta . ' )';
						}
						$item_name = strip_tags($item_name);
						$item_subtotal = number_format( $order->get_item_subtotal( $item, $include_tax ), wc_get_price_decimals(), '.', '' );
						$itemamount = $item_subtotal * $item['qty'];
						$borgun_args[ 'itemdescription_' . $item_loop ] = html_entity_decode( $item_name, ENT_NOQUOTES, 'UTF-8' );
						$borgun_args[ 'itemcount_' . $item_loop ]       = $item['qty'];
						$borgun_args[ 'itemunitamount_' . $item_loop ]  = number_format( $item_subtotal, wc_get_price_decimals(), '.', '' );
						$borgun_args[ 'itemamount_' . $item_loop ]      = number_format( $itemamount, wc_get_price_decimals(), '.', '' );
						$item_loop ++;
					}
				}
				if ( $order->get_shipping_total() > 0 ) {
					$shipping_total = $order->get_shipping_total();
					if( $include_tax ) $shipping_total += $order->get_shipping_tax();
					$shipping_total = $this->round( $shipping_total, $order );
					$borgun_args[ 'itemdescription_' . $item_loop ] = sprintf( /* translators: %s: Shipping */ __('Shipping (%s)', 'borgun_woocommerce' ), $order->get_shipping_method() );
					$borgun_args[ 'itemcount_' . $item_loop ]       = 1;
					$borgun_args[ 'itemunitamount_' . $item_loop ]  = number_format( $shipping_total, wc_get_price_decimals(), '.', '' );
					$borgun_args[ 'itemamount_' . $item_loop ]      = number_format( $shipping_total, wc_get_price_decimals(), '.', '' );
					$item_loop ++;
				}
				if (!$include_tax && $order->get_total_tax() > 0){
					$borgun_args[ 'itemdescription_' . $item_loop ] = __('Taxes', 'borgun_woocommerce' );
					$borgun_args[ 'itemcount_' . $item_loop ]       = 1;
					$borgun_args[ 'itemunitamount_' . $item_loop ]  = number_format( $order->get_total_tax(), wc_get_price_decimals(), '.', '' );
					$borgun_args[ 'itemamount_' . $item_loop ]      = number_format( $order->get_total_tax(), wc_get_price_decimals(), '.', '' );
					$item_loop ++;
				}
				if ( $order->get_total_discount() > 0 ) {
					$total_discount = $order->get_total_discount();
/*				Woocommerce can see any tax adjustments made thus far using subtotals.
					Since Woocommerce 3.2.3*/
					if(wc_tax_enabled() && method_exists('WC_Discounts','set_items') && $include_tax){
						$total_discount += $order->get_discount_tax();
					}
					if(wc_tax_enabled() && !method_exists('WC_Discounts','set_items') && !$include_tax){
						$total_discount -= $order->get_discount_tax();
					}

					$total_discount = $this->round($total_discount, $order);
					$borgun_args[ 'itemdescription_' . $item_loop ] = __('Discount', 'borgun_woocommerce' );
					$borgun_args[ 'itemcount_' . $item_loop ]       = 1;
					$borgun_args[ 'itemunitamount_' . $item_loop ]  = - number_format( $total_discount, wc_get_price_decimals(), '.', '' );
					$borgun_args[ 'itemamount_' . $item_loop ]      = - number_format( $total_discount, wc_get_price_decimals(), '.', '' );
					$item_loop ++;
				}
			}

			return $borgun_args;
		}

		//Generate the borgun button link
		function generate_borgun_form( $order_id, $redirect = true ) {
			global $woocommerce;
			if ( function_exists( 'wc_get_order' ) ) {
				$order = wc_get_order( $order_id );
			} else {
				$order = new WC_Order( $order_id );
			}

			if ( 'yes' == $this->testmode ) {
				$borgun_adr = self::BORGUN_ENDPOINT_SANDBOX;
			} else {
				$borgun_adr = self::BORGUN_ENDPOINT_LIVE;
			}
			$borgun_args       = $this->get_borgun_args( $order );
			$borgun_args_array = array();
			foreach ( $borgun_args as $key => $value ) {
				$borgun_args_array[] = '<input type="hidden" name="' . esc_attr( $key ) . '" value="' . esc_attr( $value ) . '" />';
			}

			if($redirect){
				$redirecttext = $this->get_translated_string($this->redirecttext, 'redirecttext');
				$code = sprintf( '$.blockUI({
						message: "%s",
						baseZ: 99999,
	                    overlayCSS: { background: "#fff", opacity: 0.6 },
	                    css: {
	                        padding:        "20px",
	                        zindex:         "9999999",
	                        textAlign:      "center",
	                        color:          "#555",
	                        border:         "3px solid #aaa",
	                        backgroundColor:"#fff",
	                        cursor:         "wait",
	                        lineHeight:     "24px",
	                    }
	                });

	                jQuery("#borgun_payment_form").submit();', $redirecttext );
				wc_enqueue_js( $code );
			}

			$cancel_btn_html = ( current_user_can( 'cancel_order', $order_id ) ) ? '<a class="button cancel" href="' . htmlspecialchars_decode($order->get_cancel_order_url()) . '">' . __( 'Cancel order &amp; restore cart', 'borgun_woocommerce' ) . '</a>' : '';
			$html_form = '<form action="' . esc_url( $borgun_adr ) . '" method="post" id="borgun_payment_form">'
			             . implode( '', $borgun_args_array )
			             . '<input type="submit" class="button" id="wc_submit_borgun_payment_form" value="' . __( 'Pay via Borgun', 'borgun_woocommerce' ) . '" /> ' . $cancel_btn_html . '</form>';

			return $html_form;
		}

		function process_payment( $order_id ) {
			if ( function_exists( 'wc_get_order' ) ) {
				$order = wc_get_order( $order_id );
			} else {
				$order = new WC_Order( $order_id );
			}

			return array(
				'result'   => 'success',
				'redirect' => $order->get_checkout_payment_url( true )
			);
		}

		function check_borgun_response() {
			global $woocommerce;
			global $wp;
			if( empty($_POST) ) return;
			$posted = array();
			$posted['status'] = !empty( $_POST['status'] ) ? sanitize_text_field( $_POST['status'] ) : '';
			$posted['orderid'] = !empty( $_POST['orderid'] ) ? sanitize_text_field( $_POST['orderid'] ) : '';
			$posted['reference'] = !empty( $_POST['reference'] ) ? sanitize_text_field( $_POST['reference'] ) : '';
			$posted['orderhash'] = !empty( $_POST['orderhash'] ) ? sanitize_text_field( $_POST['orderhash'] ) : '';
			$posted['step'] = !empty( $_POST['step'] ) ? sanitize_text_field( $_POST['step'] ) : '';
			$posted['errorcode'] = !empty( $_POST['errorcode'] ) ? sanitize_text_field( $_POST['errorcode'] ) : '';
			$posted['errordescription'] = !empty( $_POST['errordescription'] ) ? sanitize_text_field( $_POST['errordescription'] ) : '';
			if ( $posted['status'] == 'OK' ) {
				if ( ! empty( $posted['orderid'] ) ) {
					$order_id = (int) str_replace( 'WC-', '', $posted['orderid'] );
					if( !empty($order_id) ) {
						if ( function_exists( 'wc_get_order' ) ) {
							$order = wc_get_order( $order_id );
						} else {
							$order = new WC_Order( $order_id );
						}
						if( !empty($order) && ! $order->is_paid() ) {
							$payment_method = $order->get_payment_method();
							if( $payment_method == $this->id ) {
								$hash = $this->check_order_hash( $order );
								if ( $hash == $posted['orderhash'] ) {
									$order->add_order_note(  __( 'Borgun payment completed', 'borgun_woocommerce' ) );
									$order->payment_complete();
									$woocommerce->cart->empty_cart();
									if ( 'yes' == $this->testmode ) {
										$borgun_adr = self::BORGUN_ENDPOINT_SANDBOX;
									} else {
										$borgun_adr = self::BORGUN_ENDPOINT_LIVE;
									}
									if ( strpos( $posted['step'], 'Payment' ) !== false ) {
										$xml = '<PaymentNotification>Accepted</PaymentNotification>';
										wp_remote_post(
											$borgun_adr,
											array(
												'method'      => 'POST',
												'timeout'     => 45,
												'redirection' => 5,
												'httpversion' => '1.0',
												'headers'     => array( 'Content-Type' => 'text/xml' ),
												'body'        => array( 'postdata' => $xml, 'postfield' => 'value' ),
												'sslverify'   => false
											)
										);
									}
									if(  !empty( $this->successurl ) ){
										wp_safe_redirect( $this->successurl );
										exit;
									}
								} else {
									$order->add_order_note( __( 'Order hash doesn\'t match', 'borgun_woocommerce' ) );
									wp_safe_redirect( wc_get_checkout_url() );
									exit;
								}
							}
						}
					}
				}
			}
			elseif( $posted['status'] == 'ERROR' ) {
				$order_id = '';
				if ( ! empty( $posted['orderid'] ) ) {
					$order_id = (int) str_replace( 'WC-', '', $posted['orderid'] );
				}
				elseif ( !empty( $posted['reference'] )) {
					$order_id = (int) $posted['reference'];
				}
				elseif ( !empty( $wp->query_vars['order-received']) ) {
					$order_id = (int) $wp->query_vars['order-received'];
				}

				if( !empty($order_id) ) {
					if ( function_exists( 'wc_get_order' ) ) {
						$order = wc_get_order( $order_id );
					} else {
						$order = new WC_Order( $order_id );
					}
					if( !empty($order) ) {
						$this->error_response_process($order, $posted['status'] . ' : ' . $posted['errordescription']);
					}
				}
			}
		}

		function receipt_page( $order_id ) {
			if(!empty($_POST) && isset($_POST['status'])){
				$posted = array();
				$posted['status'] = sanitize_text_field( $_POST['status'] );
				if($posted['status'] == 'ERROR' || $posted['status'] == 'CANCEL'){
					echo $this->generate_borgun_form( $order_id, false);
				}
			}else{
				$receipttext = $this->get_translated_string($this->receipttext, 'receipttext');
				if( !empty($receipttext) ) printf('<p>%s</p>', $receipttext );
				echo $this->generate_borgun_form( $order_id);
			}
		}

		/**
		 * Round prices.
		 * @param  double $price
		 * @param  WC_Order $order
		 * @return double
		 */
		protected function round( $price, $order ) {
			$precision = 2;

			if ( ! $this->currency_has_decimals( $order->get_currency() ) ) {
				$precision = 0;
			}

			return round( $price, $precision );
		}

		/**
		 * Check if currency has decimals.
		 * @param  string $currency
		 * @return bool
		 */
		protected function currency_has_decimals( $currency ) {
			if ( in_array( $currency, array( 'HUF', 'JPY', 'TWD', 'ISK' ) ) ) {
				return false;
			}

			return true;
		}

		/**
		 * Check tax display.
		 * @return bool
		 */
		protected function tax_display() {
			$prices_include_tax = wc_tax_enabled() ? get_option( 'woocommerce_prices_include_tax' ) : 'yes';
			return ( $prices_include_tax === 'yes' ) ? true : false ;
		}

		/**
		 * Check fee tax display.
		 * @param  WC_Order_Item_Fee $item
		 * @return bool
		 */
		protected function fee_tax_display( $item ) {
			$tax_display = $item->get_tax_status();
			return ( $tax_display == 'taxable' ) ? true : false ;
		}

		/**
		 * Adds text keys to_translate.
		 * @param  $text_keys array
		 * @return array
		 */
		public function borgun_text_keys_to_translate( $text_keys ){
			if( !in_array( 'receipttext', $text_keys ) )
				$text_keys[] = 'receipttext';
			if( !in_array( 'redirecttext', $text_keys ) )
				$text_keys[] = 'redirecttext';

			return $text_keys;
		}

		/**
		 * Getting translated value
		 * @param  $string string Original field value
		 * @param  $name string Field key
		 * @return string
		 */
		public function get_translated_string( $string, $name ) {
			$translated_string = $string;
			$current_lang = apply_filters( 'wpml_current_language', NULL );
			if($current_lang && class_exists('WCML_WC_Gateways') ){
				if(defined('WCML_WC_Gateways::STRINGS_CONTEXT')){
					$domain = WCML_WC_Gateways::STRINGS_CONTEXT;
				} else {
					$domain = 'woocommerce';
				}
				$translated_string = apply_filters(
					'wpml_translate_single_string',
					$string,
					$domain,
					$this->id . '_gateway_' . $name,
					$current_lang
				);
			}

			if ( $translated_string === $string ) {
				$translated_string = __( $string, 'borgun_woocommerce' );
			}

			return $translated_string;
		}

		/**
		 * Get cancel order url
		 */
		function checkout_payment_handler(){
			global $wp;
			$order_id = '';
			if( !empty( $wp->query_vars['order-pay']) ){
				$order_id = (int) $wp->query_vars['order-pay'];
			}
			if ( $order_id && !empty($_POST) && isset($_POST['status']) ) {
				$posted = [];
				$posted['status'] = sanitize_text_field($_POST['status']);
				$posted['errordescription'] = !empty( $_POST['errordescription'] ) ? sanitize_text_field( $_POST['errordescription'] ) : '';

				if($posted['status'] === 'ERROR'){
					if ( function_exists( 'wc_get_order' ) ) {
						$order = wc_get_order( $order_id );
					} else {
						$order = new WC_Order( $order_id );
					}

					if( !empty($order) && $order->get_payment_method() == $this->id ) {
						wc_add_notice( $posted['errordescription'], 'error' );

						$this->error_response_process($order, $posted['status'] . ' : ' . $posted['errordescription']);
					}
				}
				elseif($posted['status'] === 'CANCEL'){
					wc_add_notice( __('Payment cancelled', 'borgun_woocommerce'), 'notice' );

					if ( function_exists( 'wc_get_order' ) ) {
						$order = wc_get_order( $order_id );
					} else {
						$order = new WC_Order( $order_id );
					}

					$this->cancel_response_process($order);
				}
			}
		}

		function error_response_process($order, $message = ''){
			$order->add_order_note( $message );

			if(!empty($this->errorurl)){
				wp_safe_redirect( $this->errorurl );
				exit;
			}
		}

		function cancel_response_process($order, $message = ''){
			$user_can_cancel  = current_user_can( 'cancel_order', $order->get_id() );
			if($user_can_cancel){
				$redirect = htmlspecialchars_decode($order->get_cancel_order_url());
				wp_safe_redirect( $redirect );
				exit;
			}else{
				$order->add_order_note( __('Payment canceled by the customer','borgun_woocommerce') );
				if(!empty($this->cancelurl)){
					wp_safe_redirect( $this->cancelurl );
					exit;
				}
			}
		}
	}

	add_action( 'woocommerce_cancelled_order', 'borgun_cancel_order' );
	function borgun_cancel_order( $order_id ){
		if ( function_exists( 'wc_get_order' ) ) {
			$order = wc_get_order( $order_id );
		} else {
			$order = new WC_Order( $order_id );
		}
		if( !empty($order) && $order->get_payment_method() == 'borgun' ) {
			$borgun_settings = get_option('woocommerce_borgun_settings');
			if( !empty($borgun_settings) && !empty( $borgun_settings['cancelurl'] ) ){
				if ( !empty($_POST) && isset($_POST['status']) ) {
					$posted = array();
					$posted_order_id = '';
					$posted['orderid'] = !empty( $_POST['orderid'] ) ? sanitize_text_field( $_POST['orderid'] ) : '';
					$posted['reference'] = !empty( $_POST['reference'] ) ? sanitize_text_field( $_POST['reference'] ) : '';
					if ( ! empty( $posted['orderid'] ) ) {
						$order_id = (int) str_replace( 'WC-', '', $posted['orderid'] );
					}
					elseif ( !empty( $posted['reference'] )) {
						$order_id = (int) $posted['reference'];
					}
					if($posted_order_id == $order_id){
						wp_safe_redirect( $borgun_settings['cancelurl'] );
						exit;
					}
				}else{
					wp_safe_redirect( $borgun_settings['cancelurl'] );
					exit;
				}
			}
		}
	}
}

function woocommerce_borgun_textdomain(){
	global $wp_version;

	// Default languages directory for Borgun.
	$lang_dir = BORGUN_DIR . 'languages/';
	$lang_dir = apply_filters( 'borgun_languages_directory', $lang_dir );

	$current_lang = apply_filters( 'wpml_current_language', NULL );
	if($current_lang){
		$languages = apply_filters( 'wpml_active_languages', NULL );
		$locale = ( isset($languages[$current_lang]) && isset($languages[$current_lang]['default_locale']) ) ? $languages[$current_lang]['default_locale'] : '' ;
	}else{
		$locale = get_locale();
		if ( $wp_version >= 4.7 ) {
			$locale = get_user_locale();
		}
	}

	$mofile = sprintf( '%1$s-%2$s.mo', 'borgun_woocommerce', $locale );

	// Setup paths to current locale file.
	$mofile_local  = $lang_dir . $mofile;
	$mofile_global = WP_LANG_DIR . '/plugins/' . $mofile;

	if ( file_exists( $mofile_global ) ) {
		// Look in global /wp-content/languages/borgun/ folder.
		load_textdomain( 'borgun_woocommerce', $mofile_global );
	} elseif ( file_exists( $mofile_local ) ) {
		// Look in local /wp-content/plugins/borgun/languages/ folder.
		load_textdomain( 'borgun_woocommerce', $mofile_local );
	} else {
		// Load the default language files.
		load_plugin_textdomain( 'borgun_woocommerce', false, $lang_dir );
	}
}
